@@ -3,12 +3,13 @@ Test cases for ldaptor.protocols.ldap.ldapsyntax module.
 """
 
 from twisted.trial import unittest
-from ldaptor.protocols.ldap import ldapsyntax
-from ldaptor.protocols import pureldap
+from ldaptor import config
+from ldaptor.protocols.ldap import ldapsyntax, distinguishedname, ldaperrors
+from ldaptor.protocols import pureldap, pureber
 from twisted.internet import defer
 from twisted.python import failure
 from ldaptor.testutil import LDAPClientTestDriver
-from twisted.trial.util import deferredResult
+from twisted.trial.util import deferredResult, deferredError
 
 class LDAPSyntaxBasics(unittest.TestCase):
     def testCreation(self):
@@ -171,6 +172,39 @@ class LDAPSyntaxAttributes(unittest.TestCase):
 	self.failUnlessEqual(o['bValue'], ['b'])
 	self.failUnlessEqual(o['cValue'], ['c'])
 
+    def testUndoJournaling(self):
+	"""Journaling should still work after undo."""
+        client=LDAPClientTestDriver(
+            [ pureldap.LDAPModifyResponse(resultCode=0,
+                                          matchedDN='',
+                                          errorMessage=''),
+            ])
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['a', 'b'],
+	    'aValue': ['a'],
+	    'bValue': ['b'],
+	    'cValue': ['c'],
+	    })
+	o['aValue']=['foo', 'bar']
+	o['aValue']=['quux']
+	del o['cValue']
+        o.undo()
+	o['aValue'].update(['newValue', 'anotherNewValue'])
+	d=o.commit()
+        val = deferredResult(d)
+        
+	self.failUnlessEqual(o['aValue'], ['a', 'newValue', 'anotherNewValue'])
+	self.failUnlessEqual(o['bValue'], ['b'])
+	self.failUnlessEqual(o['cValue'], ['c'])
+        client.assertSent(pureldap.LDAPModifyRequest(
+	    object='cn=foo,dc=example,dc=com',
+	    modification=[
+	    pureldap.LDAPModification_add(attributeType='aValue',
+                                          vals=['newValue', 'anotherNewValue']),
+	    ]))
+
     def testUndoAfterCommit(self):
 	"""Undo should not undo things that have been commited."""
 
@@ -407,6 +441,119 @@ class LDAPSyntaxSearch(unittest.TestCase):
 	    'bar': ['d', 'e'],
 	    }))
 
+    def testSearch_defaultAttributes(self):
+	"""Search without explicit list of attributes returns all attributes."""
+
+	client=LDAPClientTestDriver([
+            pureldap.LDAPSearchResultEntry(
+            objectName='cn=foo,dc=example,dc=com',
+            attributes=(('foo', ['a']),
+                        ('bar', ['b', 'c']),
+                        ),
+            ),
+            pureldap.LDAPSearchResultEntry(
+            objectName='cn=bar,dc=example,dc=com',
+            attributes=(('foo', ['a']),
+                        ('bar', ['d', 'e']),
+                        ),
+            ),
+            pureldap.LDAPSearchResultDone(
+            resultCode=0,
+            matchedDN='',
+            errorMessage='')
+            ])
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['organizationalUnit'],
+	    })
+
+	d=o.search(filterText='(foo=a)')
+        val = deferredResult(d)
+
+	client.assertSent(pureldap.LDAPSearchRequest(
+	    baseObject='dc=example,dc=com',
+	    scope=pureldap.LDAP_SCOPE_wholeSubtree,
+	    derefAliases=pureldap.LDAP_DEREF_neverDerefAliases,
+	    sizeLimit=0,
+	    timeLimit=0,
+	    typesOnly=0,
+	    filter=pureldap.LDAPFilter_equalityMatch(
+            attributeDesc=pureldap.LDAPAttributeDescription(value='foo'),
+            assertionValue=pureldap.LDAPAssertionValue(value='a')),
+	    attributes=[]))
+	self.failUnlessEqual(len(val), 2)
+
+	self.failUnlessEqual(val[0],
+			     ldapsyntax.LDAPEntry(
+	    client=client,
+	    dn='cn=foo,dc=example,dc=com',
+	    attributes={
+	    'foo': ['a'],
+	    'bar': ['b', 'c'],
+	    }))
+	self.failUnless(val[0].complete)
+
+	self.failUnlessEqual(val[1],
+			     ldapsyntax.LDAPEntry(
+	    client=client,
+	    dn='cn=bar,dc=example,dc=com',
+	    attributes={
+	    'foo': ['a'],
+	    'bar': ['d', 'e'],
+	    }))
+	self.failUnless(val[1].complete)
+
+    def testSearch_noAttributes(self):
+	"""Search with attributes=None returns no attributes."""
+
+	client=LDAPClientTestDriver([
+            pureldap.LDAPSearchResultEntry('cn=foo,dc=example,dc=com',
+                                           attributes=()),
+            pureldap.LDAPSearchResultEntry('cn=bar,dc=example,dc=com',
+                                           attributes=()),
+            pureldap.LDAPSearchResultDone(
+            resultCode=0,
+            matchedDN='',
+            errorMessage='')
+            ])
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['organizationalUnit'],
+	    })
+
+	d=o.search(filterText='(foo=a)',
+                   attributes=None)
+        val = deferredResult(d)
+
+	client.assertSent(pureldap.LDAPSearchRequest(
+	    baseObject='dc=example,dc=com',
+	    scope=pureldap.LDAP_SCOPE_wholeSubtree,
+	    derefAliases=pureldap.LDAP_DEREF_neverDerefAliases,
+	    sizeLimit=0,
+	    timeLimit=0,
+	    typesOnly=0,
+	    filter=pureldap.LDAPFilter_equalityMatch(
+            attributeDesc=pureldap.LDAPAttributeDescription(value='foo'),
+            assertionValue=pureldap.LDAPAssertionValue(value='a')),
+	    attributes=['1.1']))
+	self.failUnlessEqual(len(val), 2)
+
+	self.failUnlessEqual(val[0],
+			     ldapsyntax.LDAPEntry(
+	    client=client,
+	    dn='cn=foo,dc=example,dc=com'))
+	self.failIf(val[0].complete)
+
+	self.failUnlessEqual(val[1],
+			     ldapsyntax.LDAPEntry(
+	    client=client,
+	    dn='cn=bar,dc=example,dc=com'))
+	self.failIf(val[1].complete)
+
     def testSearch_ImmediateProcessing(self):
 	"""Test searches with the immediate processing feature."""
 
@@ -473,6 +620,32 @@ class LDAPSyntaxSearch(unittest.TestCase):
 	    'bar': ['b', 'c'],
 	    })])
 
+    def testSearch_fail(self):
+	client=LDAPClientTestDriver([
+            pureldap.LDAPSearchResultDone(
+            resultCode=ldaperrors.LDAPBusy.resultCode,
+            matchedDN='',
+            errorMessage='Go away')
+            ])
+
+	o=ldapsyntax.LDAPEntry(client=client, dn='dc=example,dc=com')
+	d=o.search(filterText='(foo=a)')
+        fail = deferredError(d)
+        fail.trap(ldaperrors.LDAPBusy)
+        self.assertEquals(fail.value.message, 'Go away')
+
+	client.assertSent(pureldap.LDAPSearchRequest(
+	    baseObject='dc=example,dc=com',
+	    scope=pureldap.LDAP_SCOPE_wholeSubtree,
+	    derefAliases=pureldap.LDAP_DEREF_neverDerefAliases,
+	    sizeLimit=0,
+	    timeLimit=0,
+	    typesOnly=0,
+	    filter=pureldap.LDAPFilter_equalityMatch(
+            attributeDesc=pureldap.LDAPAttributeDescription(value='foo'),
+            assertionValue=pureldap.LDAPAssertionValue(value='a')),
+            ))
+
 class LDAPSyntaxDNs(unittest.TestCase):
     def testDNKeyExistenceSuccess(self):
         client = LDAPClientTestDriver()
@@ -556,6 +729,83 @@ class LDAPSyntaxDelete(unittest.TestCase):
 	    entry='cn=foo,dc=example,dc=com',
             ))
 
+    def testErrorHandling(self):
+	"""LDAPEntry.delete should pass LDAP errors to it's deferred."""
+        client = LDAPClientTestDriver(
+            [pureldap.LDAPDelResponse(resultCode=ldaperrors.LDAPBusy.resultCode,
+                                      matchedDN='',
+                                      errorMessage='Go away'),
+             ])
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['a'],
+	    })
+	d=o.delete()
+        fail = deferredError(d)
+        fail.trap(ldaperrors.LDAPBusy)
+        self.assertEquals(fail.value.message, 'Go away')
+
+	client.assertSent(pureldap.LDAPDelRequest(
+	    entry='cn=foo,dc=example,dc=com',
+            ))
+
+    def testErrorHandling_extended(self):
+	"""LDAPEntry.delete should pass even non-LDAPDelResponse errors to it's deferred."""
+        client = LDAPClientTestDriver(
+            [pureldap.LDAPExtendedResponse(resultCode=ldaperrors.LDAPProtocolError.resultCode,
+                                                 responseName='1.3.6.1.4.1.1466.20036',
+                                                 errorMessage='Unknown request')
+             ])
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['a'],
+	    })
+	d=o.delete()
+        fail = deferredError(d)
+        fail.trap(ldaperrors.LDAPProtocolError)
+        self.assertEquals(fail.value.message, 'Unknown request')
+
+	client.assertSent(pureldap.LDAPDelRequest(
+	    entry='cn=foo,dc=example,dc=com',
+            ))
+
+class LDAPSyntaxAddChild(unittest.TestCase):
+    def testAddChildOnWire(self):
+	"""LDAPEntry.addChild should write the right data to the server."""
+        client = LDAPClientTestDriver(
+            [pureldap.LDAPAddResponse(resultCode=0,
+                                      matchedDN='',
+                                      errorMessage=''),
+             ])
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='ou=things,dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['organizationalUnit'],
+            'ou': ['things'],
+	    })
+	d=o.addChild(
+            rdn='givenName=Firstname+surname=Lastname',
+            attributes={'objectClass': ['person', 'otherStuff'],
+                        'givenName': ['Firstname'],
+                        'surname': ['Lastname'],
+                        })
+        val = deferredResult(d)
+
+	client.assertSent(pureldap.LDAPAddRequest(
+	    entry='givenName=Firstname+surname=Lastname,ou=things,dc=example,dc=com',
+            attributes=[ (pureldap.LDAPAttributeDescription('objectClass'),
+                          pureber.BERSet([pureldap.LDAPAttributeValue('person'),
+                                          pureldap.LDAPAttributeValue('otherStuff'),
+                                          ])),
+                         (pureldap.LDAPAttributeDescription('givenName'),
+                          pureber.BERSet([pureldap.LDAPAttributeValue('Firstname')])),
+                         (pureldap.LDAPAttributeDescription('surname'),
+                          pureber.BERSet([pureldap.LDAPAttributeValue('Lastname')])),
+                         ],
+            ))
+
 class LDAPSyntaxContainingNamingContext(unittest.TestCase):
     def testNamingContext(self):
 	"""LDAPEntry.namingContext returns the naming context that contains this object (via a Deferred)."""
@@ -595,6 +845,10 @@ class LDAPSyntaxContainingNamingContext(unittest.TestCase):
 
 
 class LDAPSyntaxPasswords(unittest.TestCase):
+    def setUp(self):
+        cfg = config.loadConfig()
+        cfg.set('samba', 'use-lmhash', 'no')
+
     def testPasswordSetting_ExtendedOperation(self):
         """LDAPEntry.setPassword_ExtendedOperation(newPasswd=...) changes the password."""
         client = LDAPClientTestDriver(
@@ -613,7 +867,52 @@ class LDAPSyntaxPasswords(unittest.TestCase):
             newPasswd='new'),
                         )
 
-    def testPasswordSetting_Samba(self):
+    def testPasswordSetting_Samba_sambaAccount(self):
+        """LDAPEntry.setPassword_Samba(newPasswd=...,
+        style='sambaAccount') changes the password."""
+        client = LDAPClientTestDriver(
+            [pureldap.LDAPModifyResponse(resultCode=0,
+                                         matchedDN='',
+                                         errorMessage='')],
+            )
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com')
+        d=o.setPassword_Samba(newPasswd='new', style='sambaAccount')
+        val = deferredResult(d)
+
+	client.assertSent(pureldap.LDAPModifyRequest(
+	    object='cn=foo,dc=example,dc=com',
+	    modification=[
+	    pureldap.LDAPModification_replace(attributeType='ntPassword',
+                                              vals=['89963F5042E5041A59C249282387A622']),
+	    pureldap.LDAPModification_replace(attributeType='lmPassword',
+                                              vals=['XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX']),
+            ]))
+
+    def testPasswordSetting_Samba_sambaSamAccount(self):
+        """LDAPEntry.setPassword_Samba(newPasswd=..., style='sambaSamAccount') changes the password."""
+        client = LDAPClientTestDriver(
+            [pureldap.LDAPModifyResponse(resultCode=0,
+                                         matchedDN='',
+                                         errorMessage='')],
+            )
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com')
+        d=o.setPassword_Samba(newPasswd='new', style='sambaSamAccount')
+        val = deferredResult(d)
+
+	client.assertSent(pureldap.LDAPModifyRequest(
+	    object='cn=foo,dc=example,dc=com',
+	    modification=[
+	    pureldap.LDAPModification_replace(attributeType='sambaNTPassword',
+                                              vals=['89963F5042E5041A59C249282387A622']),
+	    pureldap.LDAPModification_replace(attributeType='sambaLMPassword',
+                                              vals=['XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX']),
+            ]))
+
+    def testPasswordSetting_Samba_defaultStyle(self):
         """LDAPEntry.setPassword_Samba(newPasswd=...) changes the password."""
         client = LDAPClientTestDriver(
             [pureldap.LDAPModifyResponse(resultCode=0,
@@ -629,12 +928,29 @@ class LDAPSyntaxPasswords(unittest.TestCase):
 	client.assertSent(pureldap.LDAPModifyRequest(
 	    object='cn=foo,dc=example,dc=com',
 	    modification=[
-	    pureldap.LDAPModification_replace(attributeType='ntPassword',
+	    pureldap.LDAPModification_replace(attributeType='sambaNTPassword',
                                               vals=['89963F5042E5041A59C249282387A622']),
-	    pureldap.LDAPModification_replace(attributeType='lmPassword',
+	    pureldap.LDAPModification_replace(attributeType='sambaLMPassword',
                                               vals=['XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX']),
             ]))
 
+    def testPasswordSetting_Samba_badStyle(self):
+        """LDAPEntry.setPassword_Samba(..., style='foo') fails."""
+        client = LDAPClientTestDriver(
+            [pureldap.LDAPModifyResponse(resultCode=0,
+                                         matchedDN='',
+                                         errorMessage='')],
+            )
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com')
+        d=defer.maybeDeferred(o.setPassword_Samba, newPasswd='new', style='foo')
+        fail = deferredError(d)
+        fail.trap(RuntimeError)
+        self.assertEquals(fail.getErrorMessage(),
+                          "Unknown samba password style 'foo'")
+        client.assertNothingSent()
+
     def testPasswordSettingAll_noSamba(self):
         """LDAPEntry.setPassword(newPasswd=...) changes the password."""
         client = LDAPClientTestDriver(
@@ -821,6 +1137,41 @@ class LDAPSyntaxPasswords(unittest.TestCase):
                                        attributes=('objectClass',)),
             )
 
+    def testPasswordSetting_abortsOnFirstError(self):
+        """LDAPEntry.setPassword() aborts on first error (does not parallelize, as it used to)."""
+        client = LDAPClientTestDriver(
+            [pureldap.LDAPExtendedResponse(resultCode=ldaperrors.LDAPInsufficientAccessRights.resultCode,
+                                           matchedDN='',
+                                           errorMessage='')],
+            )
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com',
+                               attributes={
+            'objectClass': ['foo', 'sambaAccount'],
+            },
+                               complete=1)
+        d=o.setPassword(newPasswd='new')
+        fail = deferredError(d)
+        fail.trap(ldapsyntax.PasswordSetAggregateError)
+        l=fail.value.errors
+        assert len(l)==2
+
+        assert len(l[0])==2
+        self.assertEquals(l[0][0], 'ExtendedOperation')
+        assert isinstance(l[0][1], failure.Failure)
+        l[0][1].trap(ldaperrors.LDAPInsufficientAccessRights)
+
+        assert len(l[1])==2
+        self.assertEquals(l[1][0], 'Samba')
+        assert isinstance(l[1][1], failure.Failure)
+        l[1][1].trap(ldapsyntax.PasswordSetAborted)
+
+	client.assertSent(pureldap.LDAPPasswordModifyRequest(
+            userIdentity='cn=foo,dc=example,dc=com',
+            newPasswd='new'),
+                          )
+
 
 class LDAPSyntaxFetch(unittest.TestCase):
     def testFetch_WithDirtyJournal(self):
@@ -934,6 +1285,47 @@ class LDAPSyntaxFetch(unittest.TestCase):
         self.assertEquals(o['bar'], ['b', 'c'])
         self.assertEquals(o['quux'], ['baz', 'xyzzy'])
 
+    def testCommitAndFetch(self):
+	"""Fetching after a commit works."""
+
+        client = LDAPClientTestDriver(
+            [	pureldap.LDAPModifyResponse(resultCode=0,
+                                            matchedDN='',
+                                            errorMessage='')
+                ],
+            [	pureldap.LDAPSearchResultEntry('cn=foo,dc=example,dc=com',
+                                               [('aValue', ['foo', 'bar'])],
+                                               ),
+                pureldap.LDAPSearchResultDone(resultCode=0),
+                ])
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['a', 'b'],
+	    'aValue': ['a'],
+	    })
+
+	o['aValue']=['foo', 'bar']
+	d=o.commit()
+        val = deferredResult(d)
+        self.assertIdentical(o, val)
+
+	d=o.fetch('aValue')
+        val2 = deferredResult(d)
+        self.assertIdentical(o, val2)
+
+        client.assertSent(pureldap.LDAPModifyRequest(
+	    object='cn=foo,dc=example,dc=com',
+	    modification=[
+	    pureldap.LDAPModification_replace(attributeType='aValue',
+                                              vals=['foo', 'bar']),
+	    ]),
+                          pureldap.LDAPSearchRequest(
+	    baseObject='cn=foo,dc=example,dc=com',
+            scope=pureldap.LDAP_SCOPE_baseObject,
+            attributes=['aValue'],
+	    ))
+
 class LDAPSyntaxRDNHandling(unittest.TestCase):
     def testRemovingRDNFails(self):
         """Removing RDN fails with CannotRemoveRDNError."""
@@ -963,3 +1355,61 @@ class LDAPSyntaxRDNHandling(unittest.TestCase):
             o['cn']=['foo']
         self.assertRaises(ldapsyntax.CannotRemoveRDNError,
                           f)
+
+class LDAPSyntaxMove(unittest.TestCase):
+    def test_move(self):
+        client = LDAPClientTestDriver(
+            [	pureldap.LDAPModifyDNResponse(resultCode=0,
+                                              matchedDN='',
+                                              errorMessage=''),
+                ])
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com',
+                               attributes={
+	    'objectClass': ['a', 'b'],
+            'cn': ['foo'],
+	    'aValue': ['a'],
+	    })
+	d = o.move('cn=bar,ou=somewhere,dc=example,dc=com')
+        val = deferredResult(d)
+
+        client.assertSent(pureldap.LDAPModifyDNRequest(
+	    entry='cn=foo,dc=example,dc=com',
+            newrdn='cn=bar',
+            deleteoldrdn=1,
+            newSuperior='ou=somewhere,dc=example,dc=com',
+            ))
+
+        self.assertEquals(o.dn, 'cn=bar,ou=somewhere,dc=example,dc=com')
+
+class Bind(unittest.TestCase):
+    def test_ok(self):
+        client = LDAPClientTestDriver(
+            [	pureldap.LDAPBindResponse(resultCode=0,
+                                          matchedDN=''),
+                ])
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com')
+	d = defer.maybeDeferred(o.bind, 's3krit')
+        val = deferredResult(d)
+
+        client.assertSent(pureldap.LDAPBindRequest(
+	    dn='cn=foo,dc=example,dc=com',
+            auth='s3krit'))
+
+        self.assertIdentical(val, o)
+
+    def test_fail(self):
+        client = LDAPClientTestDriver(
+            [	pureldap.LDAPBindResponse(
+            resultCode=ldaperrors.LDAPInvalidCredentials.resultCode,
+            matchedDN=''),
+                ])
+
+	o=ldapsyntax.LDAPEntry(client=client,
+                               dn='cn=foo,dc=example,dc=com')
+	d = defer.maybeDeferred(o.bind, 's3krit')
+        fail = deferredError(d)
+        fail.trap(ldaperrors.LDAPInvalidCredentials)
